from enum import Enum, IntEnum

def get_from_list(input:list,index:int,default=None):
    """This is a simple function to remove index errors from the group generators"""
    try:
        return input[index]
    except IndexError:
        return default

def calculate_mjd(year: int, month: int, day:int):
    """Year: e.x 2024
    Month: (starts from 0) 0 - Jan, 1 - Feb ...
    Day - (starts from 1)"""
    l = 1 if (month == 0 or month == 1) else 0
    return (
        14956 + day + 
        int(
            ((year - 1900) - l) * 365.25
        ) +
        int(
            (month + 2 + l * 12) * 30.6001
        )
    )

def calculate_ymd(mjd:int):
    """Returns the same format as calculate_mjd, so you can encode and decode without any conversions"""
    jd = mjd + 2_400_001
    ljd = jd + 68569
    
    njd = int((4 * ljd / 146097))
    ljd = ljd - int(((146097 * njd + 3) / 4))
    
    year = int((4000 * (ljd + 1) / 1461001))
    ljd = ljd - int(((1461 * year / 4))) + 31
    
    month = int((80 * ljd / 2447))
    
    day = ljd - int((2447 * month / 80))
    
    ljd = int((month / 11));
    month = int((month + 2 - 12 * ljd))
    year = int((100 * (njd - 49) + year + ljd))
    return year, (month-1), day

class Groups(Enum):
    PS = 0
    PS_B = 1
    RT = 2
    RT_B = 3
    PTYN = 4
    ECC = 5
    LIC = 6
    TDA = 7
    TDA_B = 8
    IN_HOUSE = 9
    IN_HOUSE_B = 10

class GroupSequencer:
    """you can use this code to sequence though anything"""
    def __init__(self, sequence:list[IntEnum]) -> None:
        self.cur_idx = 1
        self.sequence = sequence
    def get_next(self):
        if len(self.sequence) == 0: return
        if self.cur_idx > len(self.sequence): self.cur_idx = 1
        prev = self.sequence[self.cur_idx-1]
        self.cur_idx += 1
        return prev
    def change_sequence(self, sequence:list[IntEnum]):
       self.sequence = sequence
       self.cur_idx = 1
    def __len__(self):
        return len(self.sequence)
    def __iter__(self):
        return self.sequence



import librds

def test_ps():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    ps = librds.GroupGenerator.ps(basic, "radio95 ", 3,ta=True)
    ps_d = dec.decode(ps)
    assert ps_d.pty == 10
    assert ps_d.tp == True
    assert ps_d.group == 0
    assert ps_d.raw_group.is_version_b == False
    
    assert ps_d.details.segment == 3
    assert ps_d.details.di == 1
    assert ps_d.details.ms == True
    assert ps_d.details.ta == True
    assert ps_d.details.text == "5 "

def test_psb():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    psb = librds.GroupGenerator.ps_b(basic, "radio95 ", 3,ta=True)
    psb_d = dec.decode(psb)
    assert psb_d.pty == 10
    assert psb_d.tp == True
    assert psb_d.group == 0
    assert psb_d.raw_group.is_version_b == True
    assert psb_d.raw_group.c == psb_d.raw_group.a
    
    assert psb_d.details.segment == 3
    assert psb_d.details.di == 1
    assert psb_d.details.ms == True
    assert psb_d.details.ta == True
    assert psb_d.details.text == "5 "

def test_rt():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    rt = librds.GroupGenerator.rt(basic,"Test\r   ",0)
    rt_d = dec.decode(rt)
    assert rt_d.pty == 10
    assert rt_d.tp == True
    assert rt_d.group == 2
    assert rt_d.raw_group.is_version_b == False
    assert rt_d.raw_group.c != rt_d.raw_group.a
    
    assert rt_d.details.segment == 0
    assert rt_d.details.ab == False
    assert rt_d.details.text == "Test"

def test_rt2():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    rt = librds.GroupGenerator.rt(basic,"Test\r   ",0,ab=True)
    rt_d = dec.decode(rt)
    assert rt_d.pty == 10
    assert rt_d.tp == True
    assert rt_d.group == 2
    assert rt_d.raw_group.is_version_b == False
    assert rt_d.raw_group.c != rt_d.raw_group.a
    
    assert rt_d.details.segment == 0
    assert rt_d.details.ab == True
    assert rt_d.details.text == "Test"

def test_rtb():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000)
    rt = librds.GroupGenerator.rt_b(basic,"Test\r   ",0)
    rt_d = dec.decode(rt)
    assert rt_d.group == 2
    assert rt_d.raw_group.is_version_b == True
    assert rt_d.raw_group.c == rt_d.raw_group.a
    
    assert rt_d.details.segment == 0
    assert rt_d.details.ab == False
    assert rt_d.details.text == "Te"

def test_rtb2():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000)
    rt = librds.GroupGenerator.rt_b(basic,"Test\r   ",0,ab=True)
    rt_d = dec.decode(rt)
    assert rt_d.group == 2
    assert rt_d.raw_group.is_version_b == True
    assert rt_d.raw_group.c == rt_d.raw_group.a
    
    assert rt_d.details.segment == 0
    assert rt_d.details.ab == True
    assert rt_d.details.text == "Te"

def test_ecc():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000)
    eccs = []
    for i in range(0xa0, 0xf3):
        eccs.append({i: librds.GroupGenerator.ecc(basic,i)})
    for ecc in eccs:
        ecc_d = dec.decode(list(ecc.values())[0])
        assert ecc_d.group == 1
        assert ecc_d.details.data == list(ecc.keys())[0]
        assert ecc_d.details.is_lic == False

def test_lic():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000)
    lics = []
    for i in range(0x0, 0x46):
        lics.append({i: librds.GroupGenerator.lic(basic,i)})
    for lic in lics:
        lic_d = dec.decode(list(lic.values())[0])
        assert lic_d.group == 1
        assert lic_d.details.data == list(lic.keys())[0]
        assert lic_d.details.is_lic == True
        
def test_ptyn():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    ptyn = librds.GroupGenerator.ptyn(basic,"Test".ljust(8),0)
    ptyn_d = dec.decode(ptyn)
    assert ptyn_d.pty == 10
    assert ptyn_d.tp == True
    assert ptyn_d.group == 10
    assert ptyn_d.raw_group.is_version_b == False
    assert ptyn_d.raw_group.c != ptyn_d.raw_group.a
    
    assert ptyn_d.details.segment == 0
    assert ptyn_d.details.ab == False
    assert ptyn_d.details.text == "Test"
def test_ptyn2():
    dec = librds.GroupDecoder()
    basic = librds.GroupGenerator.basic(0x3000, tp=True, pty=10)
    ptyn = librds.GroupGenerator.ptyn(basic,"Test".ljust(8),0,ab=True)
    ptyn_d = dec.decode(ptyn)
    assert ptyn_d.pty == 10
    assert ptyn_d.tp == True
    assert ptyn_d.group == 10
    assert ptyn_d.raw_group.is_version_b == False
    assert ptyn_d.raw_group.c != ptyn_d.raw_group.a
    
    assert ptyn_d.details.segment == 0
    assert ptyn_d.details.ab == True
    assert ptyn_d.details.text == "Test"